***********************************************************************
*                                                                     *
* MODULE NAME = BASE64                                                *
*                                                                     *
* DESCRIPTIVE NAME = Base64 encoding and decoding                     *
*                                                                     *
*    Licensed Materials - Property of IBM                             *
*                                                                     *
*    SupportPac CA8J                                                  *
*                                                                     *
*    (C) Copyright IBM Corporation 2006    All Rights Reserved.       *
*                                                                     *
*    US Government Users Restricted Rights - Use, duplication         *
*    or disclosure restricted by GSA ADP Schedule Contract            *
*    with IBM Corporation                                             *
*                                                                     *
*                                                                     *
* FUNCTION =                                                          *
*                                                                     *
*        This module provide LE/370-callable utility routines to      *
*        process base-64 encoded character strings                    *
*                                                                     *
* NOTES :                                                             *
*    DEPENDENCIES = S/390                                             *
*    RESTRICTIONS =                                                   *
*          Only callable as an Language Environment subroutine        *
*    REGISTER CONVENTIONS =                                           *
*    PATCH LABEL = None                                               *
*    MODULE TYPE = Executable                                         *
*    PROCESSOR = Assembler                                            *
*    ATTRIBUTES = Read only, Reentrant                                *
*                                                                     *
*---------------------------------------------------------------------*
*                                                                     *
* ENTRY POINT = ENCODE64                                              *
*                                                                     *
*     PURPOSE =                                                       *
*         Encode a base-64 character string                           *
*                                                                     *
*     LINKAGE =                                                       *
*         By CALL from an LE-enabled routine                          *
*                                                                     *
*     INPUT = (target, target_len, source, source_len)                *
*         target     = the string where the converted data is placed  *
*         target_len = fullword length of the target string           *
*         source     = the string to be encoded                       *
*         source_len = fullword length of the string to be encoded    *
*                                                                     *
*     OUTPUT =                                                        *
*        returned value of function is the actual length converted    *
*                                                                     *
*     EXIT-NORMAL =                                                   *
*                                                                     *
*     EXIT-ERROR =                                                    *
*                                                                     *
* ENTRY POINT = DECODE64                                              *
*                                                                     *
*     PURPOSE =                                                       *
*         Decode a base-64 character string                           *
*                                                                     *
*     LINKAGE =                                                       *
*         By CALL from an LE-enabled routine                          *
*                                                                     *
*     INPUT = (target, target_len, source, source_len)                *
*         target     = string where decoded data is placed            *
*         target_len = length of target string                        *
*         source     = base64 string to be decoded                    *
*         source_len = length of base64 string to be decoded          *
*                                                                     *
*     OUTPUT =                                                        *
*        returned value of function is the actual length converted    *
*                                                                     *
*     EXIT-NORMAL =                                                   *
*                                                                     *
*     EXIT-ERROR =                                                    *
*---------------------------------------------------------------------*
*                                                                     *
* DESCRIPTION                                                         *
*                                                                     *
*                                                                     *
***********************************************************************
&V       SETC  '1'                Version number
&R       SETC  '0'                Release number
&M       SETC  '0'                Modification number
         EJECT ,
***********************************************************************
*
*  ENCODE function
*
*  Base 64 encoding is achieved by a series of Translate instructions.
*
*  Encoding interprets successive six-bit sequences in the input
*  as numbers in the range 0..63 and represents them as eight-bit
*  characters in the base64 alphabet: A-Z,a-z,0-9,+,/.
*  The plan is to isolate the six-bit sequences into their own
*  bytes, and then translate them in a way that ignores the adjacent
*  two bits that do not form part of the six bits to be converted.
*
*  RFC2045 says: "The encoded output stream must be represented
*  in lines of no more than 76 characters each". This is achieved
*  by converting the data 54 bytes at a time, each producing a
*  72-byte result. The 72-byte segments *could* then be separated
*  by a CRLF sequence, but this is not done in this version.
*
*  The sequences could be optimized by reordering some of the
*  intermediate workareas so that fields with the same translation
*  requirements are made adjacent, and translated in one instruction,
*  but this would obscure the structure of the algorithm.
*
*  We identify the 6-bit sequences within the data to be encoded as
*  aaaaaa,bbbbbb,cccccc, etc, and byte boundaries as |.
*
*  If the initial data is:
*    aaaaaabb|bbbbcccc|ccdddddd|eeeeeeff|ffffgggg|gghhhhhh|... etc.
*
*  then a MOVE WITH OFFSET will rearrange the 6-bit sequences that
*  cross byte boundaries to place them within byte boundaries:
*    aabbbbbb|ccccccdd|ddddeeee|eeffffff|gggggghh|hhhhiiii|... etc.
*
***********************************************************************
ENCODE64 CEEENTRY MAIN=NO,PPA=ENCODPPA,                                *
               AUTO=WORKSIZE,PARMREG=R10
         USING WORKAREA,R13
         LM    R14,R1,0(R10)      Load the parameter list
         L     R15,0(,R15)        Dereference input length ptr
         L     R1,0(,R1)          Dereference output length  ptr
         STM   R14,R1,OUTPTR      Save the parameters
         SLR   R15,R15            Zero R15
         ST    R15,SIZEDONE       Initialize output length
         MVI   ENCOFFST+L'ENCOFFST-1,0 Clear final byte
ENCSTART LM    R14,R15,INPTR      Load input string ptr/len
         LTR   R15,R15            Any input?
         BNP   ENCODEOK           No, all done
         CHI   R15,L'ENCINPUT     Less than 54 bytes left?
         BNL   ENCFIXED           No, use fixed length copy
         XC    ENCINPUT,ENCINPUT  Clear input copy
         LR    R2,R15             Get copy of input length
         BCTR  R2,0               Decrement for EXECUTE
         EX    R2,ECOPYIN         Copy variable length input
         B     ENCSHIFT           Skip fixed length move
ENCFIXED LHI   R15,L'ENCINPUT     Set length to 54
         MVC   ENCINPUT,0(R14)    Copy 54 bytes from input
*
* Shift the input four bits to the left using the
* MOVE WITH OFFSET instruction (which can only move
* up to 16 bytes at a time).
*
ENCSHIFT MVO   ENCOFFST+38(16),ENCINPUT+39(15) Shift 4 bits
         MVO   ENCOFFST+23(16),ENCINPUT+24(15) Shift 4 bits
         MVO   ENCOFFST+08(16),ENCINPUT+09(15) Shift 4 bits
         MVO   ENCOFFST+00(09),ENCINPUT+00(09) Shift 4 bits
*
* Initialize selectors for the bytes in the original string
* that will become the first, second, third, and fourth
* bytes of each four-character sequence in the final result.
*
         MVC   CHAR0,SELECT1      Set up selector string
         MVC   CHAR1,SELECT1      Set up selector string
         MVC   CHAR2,SELECT2      Set up selector string
         MVC   CHAR3,SELECT3      Set up selector string
*
* Use the selectors to copy the six-bit sequences out
* of the original 24-bit sequences into their appropriate
* CHARx workareas
*
         TR    CHAR0,ENCINPUT     Select bits 0-5, 24-29,...
         TR    CHAR1,ENCOFFST     Select bits 6-11, 30-35,...
         TR    CHAR2,ENCOFFST     Select bits 12-17, 36-41,...
         TR    CHAR3,ENCINPUT     Select bits 18-23, 42-47,...
*
* The six-bit sequences are contaminated with two-bit
* sequences from other adjacent six-bit sequences.
* CHAR0 and CHAR2 are contaminated on the right, and
* CHAR1 and CHAR3 are contaminated on the left.
* The UPPERSIX and LOWERSIX translation tables are designed
* to ignore this contamination and translate only the target
* six bits.
*
         TR    CHAR0,UPPERSIX     Translate upper six bits
         TR    CHAR1,LOWERSIX     Translate lower six bits
         TR    CHAR2,UPPERSIX     Translate upper six bits
         TR    CHAR3,LOWERSIX     Translate lower six bits
*
* The translated base-64 characters are now in the wrong
* order. The following two instructions put them into the
* correct order.
*
         MVC   ENCRESLT,REORDER   Setup reordering mask
         TR    ENCRESLT,CHAR0     Put result chars in order
*
* Now calculate the length just translated and append any
* padding characters if necessary.
*
* The output is always an exact multiple of four bytes.
* If the original length divided by three has remainder
* zero, the output is exactly 4/3 of the original length.
* If the remainder is one, the extra byte is encoded as two
* base64 characters plus two padding characters ("=" signs).
* If the remainder is two, the extra two bytes are encoded
* as three base64 characters plus one padding character.
*
         LM    R0,R1,INPTR        Restore input ptr and len
         ALR   R0,R15             Adjust by length processed
         SLR   R1,R15             Adjust by length processed
         STM   R0,R1,INPTR        Save new input ptr and len
         SR    R2,R2              Clear dividend
         SR    R3,R3              Clear remainder
         IC    R2,DIVIDBY3(R15)   Input length divided by 3
         IC    R3,REMNDBY3(R15)   Remainder of divide by 3
         SLDL  R2,2               Quadruple both
         LA    R1,ENCRESLT(R2)    Offset to 4/3 of input len
         B     *+4(R3)            Process the ending
         B     ENCREM0            No remainder
         B     ENCREM1            One byte left over
         B     ENCREM2            Two bytes left over
ENCREM1  MVI   2(R1),C'='         Append first padding byte
ENCREM2  MVI   3(R1),C'='         Append final padding byte
         AHI   R1,4               Adjust ptr for final bytes
ENCREM0  EQU   *                  Skip adjust if no remainder
         LA    R0,ENCRESLT        Address start of result
         SLR   R1,R0              Calculate length of result
         LM    R14,R15,OUTPTR     Address output buffer
         CR    R1,R15             Check if enough space left
         BL    *+6                Skip next if so
         LR    R1,R15             Else just use remainder
         LR    R0,R1              Save that length
         BCTR  R1,0               Decrement for EXECUTE
         EX    R1,ECOPYOUT        Copy converted data
         ALR   R14,R0             Adjust output pointer
         SLR   R15,R0             Adjust output length
         STM   R14,R15,OUTPTR     Save adjusted values
         A     R0,SIZEDONE        Add length of current segmt
         ST    R0,SIZEDONE        Save in result length
         B     ENCSTART           Process next segment
ENCODEOK L     R1,4(,R10)         Address output length parameter
         MVC   0(4,R1),SIZEDONE   Return encoded size
         CEETERM RC=0             Return with zero return code
ECOPYIN  MVC   ENCINPUT(0),0(R14) Copy variable length input
ECOPYOUT MVC   0(0,R14),ENCRESLT  Copy encode result
         EJECT ,
***********************************************************************
*
*  DECODE function
*
*  Decode a base-64 encoded string
*
***********************************************************************
DECODE64 CEEENTRY MAIN=NO,PPA=DECODPPA,                                *
               AUTO=WORKSIZE,PARMREG=R10
         USING WORKAREA,R13
         LM    R15,R2,0(R10)      Load the parameter list
         L     R15,0(,R15)        Relocate input ptr
         L     R1,0(,R1)          Relocate output ptr
         STM   R15,R2,OUTPTR      Save the parameters
         SLR   R15,R15            Zero R15
         ST    R15,SIZEDONE       Initialize output length
         EJECT ,
*
*  Copy input string into TEMPAREA, 256 bytes at a time.
*  This is because the TRANSLATE instruction can only convert
*  256 bytes at a time.
*
DCODINIT MVI   TEMPAREA,C'='      Initialize with filler characters
         MVC   TEMPAREA+1(L'TEMPAREA-1),TEMPAREA  Propagate
         MVC   TEMPOVER,=4B'01000001'  Set delimiter bytes at end
         LM    R14,R15,INPTR      Load input string ptr and len
         LTR   R15,R15            Any input left to translate?
         BNP   DCODTERM           No, terminate
         LA    R4,256             Initialize fragment length
         CR    R4,R15             Does it exceed current length?
         BNH   *+6                No, skip next instruction
         LR    R4,R15             Yes, use current input length
         LA    R0,1               Load a constant of one
         SR    R4,R0              Decrement length for execute
         LA    R9,TEMPAREA        Address temporary workarea
*
* First check whether any characters in the input were not valid
* base-64 encoded characters. These have to be ignored. (The base-64
* encoding specification allows bursts of white space between the
* encoded characters, and it is wise to ignore all non-encoded
* characters.)
*
CHECKVAL EX    R4,VALIDATE        Check for valid characters
         BZ    COPYFRAG           All valid - copy the fragment
         SR    R1,R14             Get actual length of valid chars
         LR    R3,R1              Save for later
         BNP   SKIPFRAG           If none, ship short move
         BCTR  R1,0               Decrement for short move
         EX    R1,COPYIN          Move just the valid characters
SKIPFRAG DS    0H
         LA    R14,1(R3,R14)      Skip the non-base64 character
         AR    R9,R3              Advance workarea pointer
         SR    R4,R3              Adjust length left in TEMPAREA
         SR    R15,R3             Adjust remaining input length
         SR    R15,R0             Adjust for the invalid character
         BP    CHECKVAL           If any left, validate again
         B     SAVELOCT           Otherwise, save new locators
         EJECT ,
*
* If the remaining characters in the fragment are valid, copy them
*
COPYFRAG EX    R4,COPYIN          Copy next input fragment
         AR    R4,R0              Restore actual length
         ALR   R14,R4             Increment input pointer
         SLR   R15,R4             Decrement input length
SAVELOCT DS    0H
         STM   R14,R15,INPTR      Update input string locator
*
*  Translate each character to its corresponding 6-bit binary value.
*  Every four bytes then contain four binary numbers less than 64 that
*  represent a 24-bit binary string.
*
         TR    TEMPAREA,BASE64    Translate from base-64 to binary
         LA    R14,TEMPAREA       Point to translated string
         LA    R15,TEMPAREA       Point to translated string
         EJECT ,
*
*  The following depends on the translated value of the padding byte
*  being equal to B'01000001'. Executing BXLE R1,R1 causes R1 to be
*  added to itself and the result compared with its initial value.
*  When B'01000001' is in the high order byte, the addition causes
*  BIT1 to be moved to BIT0, and the result is negative and therefore
*  less than the initial value, and the branch is taken.
*  Non-padding values always have zeroes in both BIT0 and BIT1,
*  so the result is always a positive number equal to twice the
*  original value, and the branch is not taken.
*  (Unfortunately, the BXLE branch is also taken when R1 is zero,
*  so special checks are made for this case also.)
*
DCODTRAN DS    0H                 Start of translation loop
         LA    R2,3               Initialize decoded length
         SR    R0,R0              Zero result accumulator
         ICM   R1,15,0(R14)       Insert four translated chars
         BZ    DCODSHF0           If all zero, no bit-shuffling
         BXLE  R1,R1,DCODDONE     Shift left & branch if negative
         ALR   R1,R1              Shift left another bit
         SLDL  R0,6               Shift 6-bit translated value
         BXLE  R1,R1,DCODSHF3     Shift left & branch if negative
         ALR   R1,R1              Shift left another bit
         SLDL  R0,6               Shift 6-bit translated value
         BXLE  R1,R1,DCODSHF2     Shift left & branch if negative
         ALR   R1,R1              Shift left another bit
         SLDL  R0,6               Shift 6-bit translated value
         BXLE  R1,R1,DCODSHF1     Shift left & branch if negative
         ALR   R1,R1              Shift left another bit
         SLDL  R0,6               Shift 6-bit translated value
         B     DCODSHF0           Go save 24-bit translated result
         EJECT ,
*
*  When the delimiter is encountered, the bits decoded so far are
*  padded with zeroes to the next byte boundary.  This is done by
*  shifting in bits from the right (in multiples of six).
*  If 6 bits have been decoded, the result occupies one byte.
*            (R0 is shifted by 18 bits, R2 is decremented twice)
*  If 12 bits have been decoded, the result occupies two bytes.
*            (R0 is shifted by 12 bits, R2 is decremented once)
*  If 18 or 24 bits have been decoded, the result occupies three bytes.
*            (R0 is shifted by 6 bits if 18 bits have been decoded.
*             R2 is not decremented in either case.)
*
DCODSHF3 SLL   R0,6               Shift 6-bit result
         LTR   R1,R1              Is R1 negative (padding character)?
         BNM   DCODSHF2           No, valid zeroes present
         BCTR  R2,0               Decrement decoded length
DCODSHF2 SLL   R0,6               Shift 12-bit result
         LTR   R1,R1              Is R1 negative (padding character)?
         BNM   DCODSHF1           No, valid zeroes present
         BCTR  R2,0               Decrement decoded length
DCODSHF1 SLL   R0,6               Shift 18-bit result
         LTR   R1,R1              Is R1 negative (padding char)
         BNM   DCODSHF0           No, valid zeroes present
         BCTR  R2,0               Decrement decoded length
DCODSHF0 STCM  R0,7,0(R15)        Save 24-bit result
         LA    R15,0(R2,R15)      Step over chars just decoded
         LA    R14,4(,R14)        Step over input chars
         B     DCODTRAN           Continue translation
*
*  Calculate length just translated
*
DCODDONE DS    0H                 Base-64 to ASCII decode complete
         LA    R14,TEMPAREA       Get starting address
         LR    R0,R15             Copy ending address
         SR    R0,R14             Calculate actual string length
         LM    R14,R15,OUTPTR     Load output ptr and len
         CR    R0,R15             Converted length exceeds it?
         BNH   *+6                No, use actual converted length
         LR    R0,R15             Yes, just use length remaining
         EJECT ,
*
*  Copy the segment just translated back to caller's buffer
*
         LR    R2,R0              Save the length to copy
         BCTR  R2,0               Decrement
         EX    R2,COPYOUT         Copy translated fragment
         ALR   R14,R0             Increment output address
         SLR   R15,R0             Decrement output length
         STM   R14,R15,OUTPTR     Keep track of output position
         A     R0,SIZEDONE        Add to size done
         ST    R0,SIZEDONE        Save new value
         LTR   R15,R15            If any output length remains
         BP    DCODINIT           Restart with next segment
DCODTERM CEETERM RC=SIZEDONE      Return with size as returned value
         SPACE 2
COPYIN   MVC   0(0,R9),0(R14)     Copy input to temporaray
COPYOUT  MVC   0(0,R14),TEMPAREA  Copy temporary to output
VALIDATE TRT   0(0,R14),VALIDITY  Check for valid base-64 chars
         EJECT ,
         EJECT ,
***********************************************************************
*
*  Static values
*
***********************************************************************
         LTORG ,                       Literals
DECODPPA CEEPPA EPNAME=DECODE64,       Set up CEE static entryname     *
               INSTOP=YES,PPA2=YES,    Monitor program, generate PPA2  *
               VER=&V,REL=&R,MOD=&M    Specify version info
         SPACE 1
ENCODPPA CEEPPA EPNAME=ENCODE64,       Set up CEE static entryname     *
               INSTOP=YES,PPA2=NO,     Monitor program, omit PPA2      *
               VER=&V,REL=&R,MOD=&M    Specify version info
         SPACE 1
         DC    0D'0'              Put tables on dword boundaries
*---------------------------------------------------------------------*
*    Translate table for interpretation of base-64 encoded data.
*
*    For a description of base-64 encoding see RFC2045 at
*    http://www.ietf.org/rfc/rfc2045.txt
*---------------------------------------------------------------------*
BASE64   DC    256X'FF'
         ORG   BASE64+C'A'                       A-I
         DC    AL1(0,1,2,3,4,5,6,7,8)
         ORG   BASE64+C'J'                       J-R
         DC    AL1(9,10,11,12,13,14,15,16,17)
         ORG   BASE64+C'S'                       S-Z
         DC    AL1(18,19,20,21,22,23,24,25)
         ORG   BASE64+C'a'                       a-i
         DC    AL1(26,27,28,29,30,31,32,33,34)
         ORG   BASE64+C'j'                       j-r
         DC    AL1(35,36,37,38,39,40,41,42,43)
         ORG   BASE64+C's'                       s-z
         DC    AL1(44,45,46,47,48,49,50,51)
         ORG   BASE64+C'0'                       0-9
         DC    AL1(52,53,54,55,56,57,58,59,60,61)
         ORG   BASE64+C'+'                       +
         DC    AL1(62)
         ORG   BASE64+C'/'                       /
         DC    AL1(63)
         ORG   BASE64+C'='                       padding character
         DC    B'01000001'
         ORG   ,
*
*  Translate table for checking for the validity of the input
*  characters. Any characters not in the base-64 alphabet must
*  be ignored.

VALIDITY DC    256X'FF'
         ORG   VALIDITY+C'A'                     A-I
         DC    9AL1(0)
         ORG   VALIDITY+C'J'                     J-R
         DC    9AL1(0)
         ORG   VALIDITY+C'S'                     S-Z
         DC    8AL1(0)
         ORG   VALIDITY+C'a'                     a-i
         DC    9AL1(0)
         ORG   VALIDITY+C'j'                     j-r
         DC    9AL1(0)
         ORG   VALIDITY+C's'                     s-z
         DC    8AL1(0)
         ORG   VALIDITY+C'0'                     0-9
         DC    10AL1(0)
         ORG   VALIDITY+C'+'                     +
         DC    AL1(0)
         ORG   VALIDITY+C'/'                     /
         DC    AL1(0)
         ORG   VALIDITY+C'='                     padding character
         DC    AL1(0)
         ORG   ,
         SPACE 2
*
* This table translates an eight-bit number to the EBCDIC
* base-64 character implied by its low-order six bits:
* that is, it is insensitive to the high-order two bits.
*
LOWERSIX DC    C'ABCDEFGHIJKLMNOPQRSTUYWXYZabcdef'
         DC    C'ghijklmnopqrstuvwxyz0123456789+/'
         DC    C'ABCDEFGHIJKLMNOPQRSTUYWXYZabcdef'
         DC    C'ghijklmnopqrstuvwxyz0123456789+/'
         DC    C'ABCDEFGHIJKLMNOPQRSTUYWXYZabcdef'
         DC    C'ghijklmnopqrstuvwxyz0123456789+/'
         DC    C'ABCDEFGHIJKLMNOPQRSTUYWXYZabcdef'
         DC    C'ghijklmnopqrstuvwxyz0123456789+/'
         SPACE 2
*
* This table translates an eight-bit number to the EBCDIC
* base-64 character implied by its high-order six bits:
* that is, it is insensitive to the low-order two bits.
*
UPPERSIX DC    C'AAAABBBBCCCCDDDDEEEEFFFFGGGGHHHH'
         DC    C'IIIIJJJJKKKKLLLLMMMMNNNNOOOOPPPP'
         DC    C'QQQQRRRRSSSSTTTTUUUUVVVVWWWWXXXX'
         DC    C'YYYYZZZZaaaabbbbccccddddeeeeffff'
         DC    C'gggghhhhiiiijjjjkkkkllllmmmmnnnn'
         DC    C'ooooppppqqqqrrrrssssttttuuuuvvvv'
         DC    C'wwwwxxxxyyyyzzzz0000111122223333'
         DC    C'444455556666777788889999++++////'
         SPACE 2
*
*  These selectors select the bytes in the original string
*  that will be moved to CHAR0, CHAR1, CHAR2, and CHAR3
*  respectively.
*
SELECT1  DC    AL1(0,3,6,09,12,15,18,21,24,27,30,33,36,39,42,45,48,51)
SELECT2  DC    AL1(1,4,7,10,13,16,19,22,25,28,31,34,37,40,43,46,49,52)
SELECT3  DC    AL1(2,5,8,11,14,17,20,23,26,29,32,35,38,41,44,47,50,53)

         SPACE 2
*
*  This table moves the bytes in CHAR0, CHAR1, CHAR2, and CHAR3
*  into their correct positions in the final output string.
*
REORDER  DC    AL1(00,18,36,54)
         DC    AL1(01,19,37,55)
         DC    AL1(02,20,38,56)
         DC    AL1(03,21,39,57)
         DC    AL1(04,22,40,58)
         DC    AL1(05,23,41,59)
         DC    AL1(06,24,42,60)
         DC    AL1(07,25,43,61)
         DC    AL1(08,26,44,62)
         DC    AL1(09,27,45,63)
         DC    AL1(10,28,46,64)
         DC    AL1(11,29,47,65)
         DC    AL1(12,30,48,66)
         DC    AL1(13,31,49,67)
         DC    AL1(14,32,50,68)
         DC    AL1(15,33,51,69)
         DC    AL1(16,34,52,70)
         DC    AL1(17,35,53,71)
*
*  Tables for a fast divide by three for numbers below 60
*
DIVIDBY3 DC    AL1(0,0,0,1,1,1,2,2,2,3,3,3,4,4,4,5,5,5)
         DC    AL1(6,6,6,7,7,7,8,8,8,9,9,9,10,10,10,11,11,11)
         DC    AL1(12,12,12,13,13,13,14,14,14,15,15,15)
         DC    AL1(16,16,16,17,17,17,18,18,18,19,19,19)
         SPACE 3
REMNDBY3 DC    AL1(0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2)
         DC    AL1(0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2)
         DC    AL1(0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2)
         DC    AL1(0,1,2,0,1,2,0,1,2,0,1,2,0,1,2,0,1,2)
         SPACE 3
         EJECT ,
*
* Mapping macros
*
         CEECAA ,                 Mapping of CEE Common Anchor Aeea
         CEEDSA ,                 Mapping of CEE Dynamic Storage Area
WORKAREA DSECT ,
         DS    XL(CEEDSASZ)       Space for CEE DSA information
OUTPTR   DS    A                  Current output position
OUTLEN   DS    A                  Remaining output length
INPTR    DS    A                  Current input position
INLEN    DS    A                  Remaining input length
SIZEDONE DS    F                  Output size actually converted
         DS    A                  Alignment
TEMPAREA DS    CL256              Translation workarea
TEMPOVER DS    CL4                Overflow Area (for delimiter)
         ORG   TEMPAREA
ENCRESLT DS    CL72               Final ENCODE result
ENCINPUT DS    CL54               Input to be encoded
ENCOFFST DS    CL54               Input offset left by 4 bits
         SPACE 1
CHAR0    DS    CL18               All CHAR0 bytes
CHAR1    DS    CL18               All CHAR1 bytes
CHAR2    DS    CL18               All CHAR2 bytes
CHAR3    DS    CL18               All CHAR3 bytes
         SPACE 1
WORKSIZE EQU   *-WORKAREA
         YREGS ,                  Register equates
ENCODE64 CSECT ,                  Resume program csect
         DC    C'Licensed Materials - Property of IBM.'
         DC    C' (C) Copyright IBM Corporation 2006'
         DC    C' All Rights Reserved.'
         DC    C' US Government Users Restricted Rights -'
         DC    C' Use, duplication or disclosure restricted'
         DC    C' by GSA ADP Schedule Contract with IBM Corporation'
         END