CBL CICS('COBOL3') APOST
      *****************************************************************
      *
      *  MODULE NAME = BASIC230
      *
      *  DESCRIPTIVE NAME = HTTP Client Basic Authentication program
      *
      *  Licensed Materials - Property of IBM
      *
      *  SupportPac CA8J
      *
      *  (C) Copyright IBM Corporation 2006    All Rights Reserved.
      *
      *  US Government Users Restricted Rights - Use, duplication
      *  or disclosure restricted by GSA ADP Schedule Contract
      *  with IBM Corporation
      *
      *  FUNCTION =
      *     This program prepares and sends an HTTP
      *     Basic Authentication header.
      *     It generates the header by calling HTTPAUTH, and then
      *     sends it using EXEC CICS WRITE HTTPHEADER.
      *
      *     This sample requires CICS Transaction Server 2.2 or 2.3.
      *
      *  NOTES :
      *   DEPENDENCIES = S/390
      *   RESTRICTIONS =
      *   PATCH LABEL = None
      *   MODULE TYPE = Executable
      *   PROCESSOR = COBOL
      *   ATTRIBUTES = Read only, Serially Reusable
      *
      ******************************************************************
       IDENTIFICATION DIVISION.
       PROGRAM-ID.    BASIC230.
       DATE-WRITTEN.  December 2005.
       DATE-COMPILED. December 2005.
      ******************************************************************
      ****                                                          ****
      ****        B A S I C   A U T H E N T I C A T I O N           ****
      ****                                                          ****
      ****                                                          ****
      ****  This program uses Assembler subroutines to create       ****
      ****  the contents of an HTTP Basic Authentication header,    ****
      ****  and then uses CICS Web Support API to add it to         ****
      ****  the headers for an outbound HTTP request.               ****
      ****                                                          ****
      ******************************************************************

       ENVIRONMENT DIVISION.

       DATA DIVISION.

       WORKING-STORAGE SECTION.
        COPY DFHWBCLO.

        77 HTTP-SESSION            PIC X(8) VALUE LOW-VALUES.
        77 HTTP-RESPONSE-PTR       USAGE IS POINTER.
        77 HTTP-RESPONSE-LENGTH    PIC S9(8) COMP-5.
        77 HTTP-STATUS-CODE        PIC 9999  COMP-5.
        77 HTTP-STATUS-TEXT        PIC X(80).
        77 HTTP-MEDIATYPE          PIC X(56).
        77 BASIC-AUTH-VALUE-LENGTH PIC S9(8) COMP-5 VALUE ZERO.

        01 TARGET-URL.
           02 HTTP-SCHEME  PIC X(7)  VALUE 'http://'.
           02 HTTP-HOST    PIC X(24) VALUE 'winmvs2c.hursley.ibm.com'.
           02 HTTP-PORT    PIC X(5)  VALUE ':5053'.
           02 HTTP-PATH    PIC X(22) VALUE '/cics/cwba/dfh0wbca'.

        01 HTTPAUTH-PARAMETERS.
           02 HTTPAUTH-FUNCTION        PIC S9(8) COMP-5 VALUE IS 1.
           02 HTTPAUTH-USERID-LENGTH   PIC S9(8) COMP-5.
           02 HTTPAUTH-PASSWORD-LENGTH PIC S9(8) COMP-5.
           02 HTTPAUTH-USERID          PIC X(8)  VALUE 'UUUUUUUU'.
           02 HTTPAUTH-PASSWORD        PIC X(8)  VALUE 'PPPPPPPP'.

        01 HTTP-HEADERS.
           02 MISCELLANEOUS-HEADER.
              03 MISC-HDR-NAME     PIC X(17) VALUE 'X-Miscellaneous: '.
              03 MISC-HDR-VALUE    PIC X(2)  VALUE '""'.
              03 FILLER            PIC X(2)  VALUE X'0D25'.
           02 BASIC-AUTHENTICATION-HEADER.
              03 BASIC-AUTH-NAME   PIC X(15)  VALUE 'Authorization: '.
              03 BASIC-AUTH-VALUE  PIC X(256) VALUE ALL '-'.

        01 HTTP-ERROR-RESPONSE.
           02 FILLER    PIC X(16) VALUE 'DFHWBCLI failed.'.
           02 FILLER    PIC X(10) VALUE ' RESPONSE='.
           02 ERR-RESP  PIC 999.
           02 FILLER    PIC X(8)  VALUE ' REASON='.
           02 ERR-REAS  PIC 999.
           02 FILLER    PIC X(13) VALUE ' HTTP STATUS='.
           02 HTTP-STAT PIC 999.

       LINKAGE SECTION.
        01 HTTP-RESPONSE.
           02 FILLER PIC X OCCURS 1 TO 32767
                           DEPENDING ON HTTP-RESPONSE-LENGTH.

        01 WBCLI-BODY.
           02 FILLER PIC X OCCURS 1 TO 32767
                           DEPENDING ON WBCLI-RESPONSE-BODY-LEN.

       PROCEDURE DIVISION.
       MAINLINE SECTION.
      *
      * Link to the CICS Web Client Utility program to determine
      * the default proxy URL, if any.
      * If required, the proxy is specified in SIT override:
      *  INITPARM=(DFHWBCLI,'PROXY=http://....')
      * If the proxy is found, the DFHWBCLI parameter list is
      * set up to use it: WBCLI-PROXY-URL-PTR, WBCLI-PROXY-URL-LEN,
      * and WBCLI-USE-PROXY are set in the parameter list.
      *
           MOVE LOW-VALUES TO DFHWBCLI-ARG.
           SET  WBCLI-SET-RESP-BUFFER TO TRUE.
           MOVE WBCLI-VERSION-CURRENT TO WBCLI-VERSION-NO.
           MOVE WBCLI-FUNCTION-INQUIRE-PROXY TO WBCLI-FUNCTION.
           EXEC CICS LINK PROGRAM('DFHWBCLI')
                          COMMAREA(DFHWBCLI-ARG)
                          LENGTH(LENGTH OF DFHWBCLI-ARG)
           END-EXEC.
      *
      * Set up the Basic Authentication header value.
      *
           MOVE LENGTH OF BASIC-AUTH-VALUE  TO BASIC-AUTH-VALUE-LENGTH.
           MOVE LENGTH OF HTTPAUTH-USERID   TO HTTPAUTH-USERID-LENGTH.
           MOVE LENGTH OF HTTPAUTH-PASSWORD TO HTTPAUTH-PASSWORD-LENGTH.
           CALL 'HTTPAUTH' USING HTTPAUTH-FUNCTION
                           BASIC-AUTH-VALUE
                           BASIC-AUTH-VALUE-LENGTH
                           HTTPAUTH-USERID
                           HTTPAUTH-USERID-LENGTH
                           HTTPAUTH-PASSWORD
                           HTTPAUTH-PASSWORD-LENGTH.
      *
      * Set pointer & length of all the required headers,
      * including the Basic Auth "Authorization" header,
      * plus any arbitrary "miscellaneous" ones.
      * (These could include SOAPaction if this was a SOAP sample.)
      *
           SET WBCLI-HEADER-PTR TO ADDRESS OF HTTP-HEADERS.
           COMPUTE WBCLI-HEADER-LEN =
                   LENGTH OF HTTP-HEADERS -
                   LENGTH OF BASIC-AUTH-VALUE +
                   BASIC-AUTH-VALUE-LENGTH.
      *
      * Set up remaining DFHWBCLI parameters for an HTTP GET request.
      *
           MOVE LENGTH OF TARGET-URL TO WBCLI-URL-LEN.
           SET WBCLI-URL-PTR TO ADDRESS OF TARGET-URL.
           MOVE WBCLI-FUNCTION-CONVERSE TO WBCLI-FUNCTION.
           MOVE WBCLI-METHOD-GET        TO WBCLI-METHOD.
      *
      * Link to the CICS Web Client Utility program.
      *
           EXEC CICS LINK PROGRAM('DFHWBCLI')
                          COMMAREA(DFHWBCLI-ARG)
                          LENGTH(LENGTH OF DFHWBCLI-ARG)
           END-EXEC.

           IF WBCLI-RESPONSE = WBCLI-RESPONSE-OK THEN
              PERFORM COPY-HTTP-RESPONSE
           ELSE
              PERFORM SET-HTTP-ERROR-RESPONSE
           END-IF.
      *
      * Display up to 24 lines of the response
      *
           IF HTTP-RESPONSE-LENGTH > 1920 THEN
              MOVE 1920 TO HTTP-RESPONSE-LENGTH.

           EXEC CICS SEND ERASE FROM(HTTP-RESPONSE)
                                FLENGTH(HTTP-RESPONSE-LENGTH)
                                END-EXEC.
      *
      * Return
      *
           EXEC CICS RETURN
           END-EXEC.

      *****************************************************************
      *
      * Subroutine to copy response and remove non-display characters.
      *
      *****************************************************************
       COPY-HTTP-RESPONSE.
           MOVE WBCLI-RESPONSE-BODY-LEN TO HTTP-RESPONSE-LENGTH.

           EXEC CICS GETMAIN SET(HTTP-RESPONSE-PTR)
                             FLENGTH(HTTP-RESPONSE-LENGTH)
                             END-EXEC.

           SET ADDRESS OF WBCLI-BODY    TO WBCLI-RESPONSE-BODY-PTR.
           SET ADDRESS OF HTTP-RESPONSE TO HTTP-RESPONSE-PTR.
           MOVE WBCLI-BODY TO HTTP-RESPONSE.
           INSPECT HTTP-RESPONSE
                   CONVERTING X'0D251505' TO X'1E1E1E40'.
      *****************************************************************
      *
      *  Subroutine to create an error response.
      *
      *****************************************************************
       SET-HTTP-ERROR-RESPONSE.
           MOVE WBCLI-RESPONSE         TO ERR-RESP.
           MOVE WBCLI-REASON           TO ERR-REAS.
           MOVE WBCLI-HTTP-STATUS-CODE TO HTTP-STAT.
           MOVE LENGTH OF HTTP-ERROR-RESPONSE TO HTTP-RESPONSE-LENGTH.
           EXEC CICS GETMAIN SET(HTTP-RESPONSE-PTR)
                             FLENGTH(HTTP-RESPONSE-LENGTH)
                             END-EXEC.
           SET ADDRESS OF HTTP-RESPONSE TO HTTP-RESPONSE-PTR.
           MOVE HTTP-ERROR-RESPONSE TO HTTP-RESPONSE.