***********************************************************************
*                                                                     *
* MODULE NAME = HTTPAUTH                                              *
*                                                                     *
* DESCRIPTIVE NAME = Create HTTP authorization header text            *
*                                                                     *
*                                                                     *
*    Licensed Materials - Property of IBM                             *
*                                                                     *
*    SupportPac CA8J                                                  *
*                                                                     *
*    (C) Copyright IBM Corporation 2006    All Rights Reserved.       *
*                                                                     *
*    US Government Users Restricted Rights - Use, duplication         *
*    or disclosure restricted by GSA ADP Schedule Contract            *
*    with IBM Corporation                                             *
*                                                                     *
*                                                                     *
* FUNCTION =                                                          *
*                                                                     *
*        This module provide an LE-callable utility routines to       *
*        create the header value for an HTTP Basic Authentication     *
*        header.                                                      *
*                                                                     *
* NOTES :                                                             *
*    DEPENDENCIES = S/390                                             *
*    RESTRICTIONS =                                                   *
*          Only callable as an Language Environment subroutine        *
*    REGISTER CONVENTIONS =                                           *
*    PATCH LABEL = None                                               *
*    MODULE TYPE = Executable                                         *
*    PROCESSOR = Assembler                                            *
*    ATTRIBUTES = Read only, Reentrant                                *
*                                                                     *
*---------------------------------------------------------------------*
*                                                                     *
* ENTRY POINT = HTTPAUTH                                              *
*                                                                     *
*     PURPOSE =                                                       *
*         Create Authorization header text                            *
*                                                                     *
*     LINKAGE =                                                       *
*         By CALL from an LE-enabled routine                          *
*                                                                     *
*     INPUT = (function_code,                                         *
*              target,        target_len,                             *
*              userid,        userid_len,                             *
*              password,      password_len)                           *
*       function_code = fullword function code number                 *
*       target        = address at which converted string is placed   *
*       target_len    = length of target string                       *
*       userid        = address of userid to be encoded               *
*       userid_len    = length of userid to be encoded                *
*       password      = address of password to be encoded             *
*       password_len  = length of password to be encoded              *
*                                                                     *
*       Since the userid and password may be on a non-z/OS system,    *
*       no assumption is made about their length, except that         *
*       the string 'userid:password' cannot exceed 256 bytes.         *
*                                                                     *
*     OUTPUT =                                                        *
*        returned value of function is the actual length converted    *
*                                                                     *
*                                                                     *
***********************************************************************
&V       SETC  '1'                Version number
&R       SETC  '0'                Release number
&M       SETC  '0'                Modification number
         EJECT ,
*
*  Decode a base-64 encoded string
*
HTTPAUTH CEEENTRY MAIN=NO,PPA=HTTPAPPA,                                *
               AUTO=WORKSIZE,PARMREG=R10
         USING STORAGE,R13
         LM    R15,R5,0(R10)      Load parameter list pointers
         STM   R15,R5,PARMLIST    Save in local storage
         ST    R0,OUTPTR          Save output pointer
         MVC   FUNCODE,0(R15)     Save function code
         MVC   OUTLEN,0(R1)       Save target length
         MVC   USRIDLEN,0(R3)     Save userid length
         MVC   PSWDLEN,0(R5)      Save password length
         SPACE 1
         MVI   AUTHENT,C' '       Initialize authenticator
         MVC   AUTHENT+1(L'AUTHENT-1),AUTHENT ... to spaces
         LA    R9,L'AUTHENT       Get space left in output
         L     R0,USRIDPTR        Get pointer to userid
         L     R1,USRIDLEN        Get length of userid
         LA    R14,AUTHENT        Address space for userid:password
         LR    R15,R1             Copy userid length
         CR    R15,R9             More than space left?
         BNH   *+6                No, skip next
         LR    R15,R9             Yes, just use space left
         SR    R9,R15             Decrement space left
         MVCL  R14,R0             Copy userid to workarea
         LA    R15,AUTHENT        Address start of string again
         LA    R0,C' '            Set up search character
         SRST  R14,R15            Search for trailing blank
         LA    R9,AUTHENT+L'AUTHENT Address end of authenticator
         SLR   R9,R14             Recalculate space left
         AHI   R9,-1              Decrement space left
         BNP   B64NCODE           If none left, skip to encode result
         MVI   0(R14),C':'        Insert delimiter
         AHI   R14,1              Bump over it
         L     R0,PSWDPTR         Get pointer to password
         L     R1,PSWDLEN         Get length of password
         LR    R15,R1             Copy its length
         CR    R15,R9             More than space left?
         BNH   *+6                No, skip next
         LR    R15,R9             Yes, just use space left
         SR    R9,R15             Decrement space left
         MVCL  R14,R0             Copy password to workarea
         LA    R15,AUTHENT        Address start of string again
         LA    R0,C' '            Set up search character
         SRST  R14,R15            Search for trailing blank
         LA    R9,AUTHENT+L'AUTHENT Address end of authenticator
         SLR   R9,R14             Recalculate space left
         SPACE 2
B64NCODE LA    R0,L'AUTHENT       Get total length of authenticator
         SR    R0,R9              Subtract space left to get space used
         ST    R0,AUTHLEN         Save actual length of authenticator
         TR    AUTHENT,ASCII      Translate authenticator to ASCII
         LA    R0,L'B64AUTH       Get length of base64 space
         ST    R0,B64AULEN        Save as output length
         LA    R14,B64AUTH        Address base64 authenticator
         LA    R15,B64AULEN       Address base64 length
         LA    R0,AUTHENT         Address authenticator (now in ASCII)
         LA    R1,AUTHLEN         Address its length
         STM   R14,R1,B64PLIST    Save pointers in B64 parmlist
         OI    B64INLEN,X'80'     Set VL bit in parmlist
         LA    R1,B64PLIST        Reset R1 to parmlist
         L     R15,VB64ENC        Address Base64 encode routine
         BASR  R14,R15            Call Base64 encode routine
         LM    R0,R1,OUTPTR       Address output buffer
         MVC   AUTOKEN,BASIC      Set 'Basic' token in response
         LA    R14,RESPONSE       Address response
         L     R15,B64AULEN       Get length of converted authenticator
         AHI   R15,L'BASIC        Add length of basic token
         L     R2,TARGLPTR        Address the length parameter
         ST    R15,0(,R2)         Return authenticator length
         MVCL  R0,R14             Copy authenticator back to caller
         CEETERM RC=0             Return to caller
         EJECT ,
HTTPAPPA CEEPPA EPNAME=HTTPAUTH,       Set up CEE static entryname     *
               INSTOP=YES,PPA2=YES,    Monitor program, generate PPA2  *
               VER=&V,REL=&R,MOD=&M    Specify version info
         SPACE 1
VB64ENC  DC    V(ENCODE64)        Address of Base64 encode routine
BASIC    DC    C'Basic '          Token for Basic Authentication
*
*  The following translates EBCDIC codepage 037 to ISO-8859-1
*  If you use another EBCDIC codepage, you may need to change this
*
ASCII    DC    0CL256' '          EBCDIC to ASCII translate table
         DC    X'000102039C09867F978D8E0B0C0D0E0F'
         DC    X'101112139D8508871819928F1C1D1E1F'
         DC    X'80818283840A171B88898A8B8C050607'
         DC    X'909116939495960498999A9B14159E1A'
         DC    X'20A0E2E4E0E1E3E5E7F1A22E3C282B7C'
         DC    X'26E9EAEBE8EDEEEFECDF21242A293BAC'
         DC    X'2D2FC2C4C0C1C3C5C7D1A62C255F3E3F'
         DC    X'F8C9CACBC8CDCECFCC603A2340273D22'
         DC    X'D8616263646566676869ABBBF0FDFEB1'
         DC    X'B06A6B6C6D6E6F707172AABAE6B8C6A4'
         DC    X'B57E737475767778797AA1BFD0DDDEAE'
         DC    X'5EA3A5B7A9A7B6BCBDBE5B5DAFA8B4D7'
         DC    X'7B414243444546474849ADF4F6F2F3F5'
         DC    X'7D4A4B4C4D4E4F505152B9FBFCF9FAFF'
         DC    X'5CF7535455565758595AB2D4D6D2D3D5'
         DC    X'30313233343536373839B3DBDCD9DA9F'
         LTORG ,                  Literals
*
* Mapping macros
*
         CEECAA ,                 Mapping of CEE Common Anchor Aeea
         CEEDSA ,                 Mapping of CEE Dynamic Storage Area
STORAGE  DSECT ,
         DS    XL(CEEDSASZ)       Space for CEE DSA information
PARMLIST EQU   *
FUNCPTR  DS    A                  Address of function code
TARGPTR  DS    A                  Address of target buffer
TARGLPTR DS    A                  Address of target buffer length
USRIDPTR DS    A                  Address of userid
USRLPTR  DS    A                  Address of userid length
PSWDPTR  DS    A                  Address of password
PSWDLPTR DS    A                  Address of password length
PARMLEN  EQU   *-PARMLIST         Expected length of parmlist
 space 2
B64PLIST DS    0A
B64OUPTR DS    A                  Address of Base64 output
B64OULEN DS    A                  Address of Base64 output length
B64INPTR DS    A                  Address of Base64 input
B64INLEN DS    A                  Address of Base64 input length
         SPACE 2
FUNCODE  DS    F                  Function code
OUTPTR   DS    A                  Address of output
OUTLEN   DS    F                  Length of output
USRIDLEN DS    F                  Length of input userid
PSWDLEN  DS    F                  Length of input password
AUTHLEN  DS    F                  Length of AUTHENT
B64AULEN DS    F                  Length of B64AUTH
RESPONSE EQU   *
AUTOKEN  DS    CL(L'BASIC)        Basic auth keyword
B64AUTH  DS    CL((L'AUTHENT+2)*4/3) Base64 encoded userid:password
AUTHENT  DS    CL256              Unencoded userid:password
         SPACE 1
WORKSIZE EQU   *-STORAGE
         YREGS ,                  Register equates
HTTPAUTH CSECT ,                  Resume program csect
         DC    C'Licensed Materials - Property of IBM.'
         DC    C' (C) Copyright IBM Corporation 2006'
         DC    C' All Rights Reserved.'
         DC    C' US Government Users Restricted Rights -'
         DC    C' Use, duplication or disclosure restricted'
         DC    C' by GSA ADP Schedule Contract with IBM Corporation'
         END