CBL CICS('COBOL3') APOST
      *****************************************************************
      *
      *  MODULE NAME = WSBASIC
      *
      *  DESCRIPTIVE NAME = Basic Authentication transport handler
      *
      *  Licensed Materials - Property of IBM
      *
      *  SupportPac CA8J
      *
      *  (C) Copyright IBM Corporation 2006    All Rights Reserved.
      *
      *  US Government Users Restricted Rights - Use, duplication
      *  or disclosure restricted by GSA ADP Schedule Contract
      *  with IBM Corporation
      *
      *  FUNCTION =
      *     This program acts as an HTTP transport handler in a
      *     CICS Web Services pipeline.
      *     Its purpose is to insert a Basic Authentication
      *     HTTP header, by calling the HTTPAUTH service.
      *
      *  NOTES :
      *   DEPENDENCIES = S/390
      *   RESTRICTIONS =
      *   PATCH LABEL = None
      *   MODULE TYPE = Executable
      *   PROCESSOR = COBOL
      *   ATTRIBUTES = Read only, Serially Reusable
      *
      *
      ******************************************************************
       IDENTIFICATION DIVISION.
       PROGRAM-ID.    WSBASIC.
       DATE-WRITTEN.  December 2005.
       DATE-COMPILED. October 2006.
      ******************************************************************
      ****                                                          ****
      ****        B A S I C   A U T H E N T I C A T I O N           ****
      ****           T R A N S P O R T   H A N D L E R              ****
      ****                                                          ****
      ****                                                          ****
      ****  This program uses Assembler subroutines to create       ****
      ****  the contents of an HTTP Basic Authentication header.    ****
      ****  The pipeline architecture does not formally allow the   ****
      ****  insertion of arbitrary HTTP headers into the output     ****
      ****  message, but it does allow the specification of a       ****
      ****  SOAPAction header. We spoof the pipeline into adding    ****
      ****  a Basic Authentication header by appending it to the    ****
      ****  value field of the SOAPAction header.                   ****
      ****                                                          ****
      ******************************************************************

       ENVIRONMENT DIVISION.

       DATA DIVISION.

       WORKING-STORAGE SECTION.
        77 HANDLER-ACTION            PIC X(16).
        77 SOAPACTION-CONTAINER-SIZE PIC S9(8) COMP-5 SYNC VALUE ZERO.
        77 APPEND-LENGTH             PIC S9(8) COMP-5 SYNC VALUE ZERO.
        77 BASIC-AUTH-VALUE-LENGTH   PIC S9(8) COMP-5 SYNC VALUE ZERO.

        01 HTTPAUTH-PARAMETERS.
           02 HTTPAUTH-FUNCTION        COMP-5 PIC S9(8) VALUE IS 1.
           02 HTTPAUTH-USERID-LENGTH   COMP-5 PIC S9(8) VALUE IS 7.
           02 HTTPAUTH-PASSWORD-LENGTH COMP-5 PIC S9(8) VALUE IS 8.
           02 HTTPAUTH-USERID          PIC X(8) VALUE 'UUUUUUUU'.
           02 HTTPAUTH-PASSWORD        PIC X(8) VALUE 'PPPPPPPP'.

        01 SOAPACTION-CONTAINER.
           02 SOAPACTION-CHAR PIC X OCCURS 2048.

        01 BASIC-AUTHENTICATION-HEADER.
           02 FILLER           PIC XX     VALUE X'0D25'.
           02 BASIC-AUTH-NAME  PIC X(15)  VALUE 'Authorization: '.
           02 BASIC-AUTH-VALUE PIC X(256) VALUE ALL '-'.

       LINKAGE SECTION.
        01 SOAPACTION-APPENDAGE.
           02 FILLER PIC X OCCURS 1 TO 1024
                           DEPENDING ON APPEND-LENGTH.

       PROCEDURE DIVISION.
       MAINLINE SECTION.
      *
      * Find out why we are called by getting the DFHFUNCTION container
      *
           MOVE SPACES TO HANDLER-ACTION.
           EXEC CICS GET CONTAINER('DFHFUNCTION')
                         INTO(HANDLER-ACTION)
                         NOHANDLE END-EXEC.
           EVALUATE HANDLER-ACTION
             WHEN 'SEND-REQUEST'
               PERFORM SETUP-BASIC-AUTHENTICATION
             WHEN 'SEND-RESPONSE'
               PERFORM DELETE-REQUEST-CONTAINER
             WHEN OTHER
               CONTINUE
           END-EVALUATE.
      * Return
           EXEC CICS RETURN
           END-EXEC.

      * Set up the Basic Authentication header value
       SETUP-BASIC-AUTHENTICATION SECTION.
           MOVE LENGTH OF BASIC-AUTH-VALUE TO BASIC-AUTH-VALUE-LENGTH.
           CALL 'HTTPAUTH' USING HTTPAUTH-FUNCTION
                           BASIC-AUTH-VALUE
                           BASIC-AUTH-VALUE-LENGTH
                           HTTPAUTH-USERID
                           HTTPAUTH-USERID-LENGTH
                           HTTPAUTH-PASSWORD
                           HTTPAUTH-PASSWORD-LENGTH.
      *
      * Get the current contents of the SOAPaction container,
      * or use "" if none is present.
      *
           MOVE LENGTH OF SOAPACTION-CONTAINER
                       TO SOAPACTION-CONTAINER-SIZE.
           EXEC CICS GET CONTAINER('DFHWS-SOAPACTION')
                         INTO(SOAPACTION-CONTAINER)
                         FLENGTH(SOAPACTION-CONTAINER-SIZE)
                         NOHANDLE END-EXEC.

           IF SOAPACTION-CONTAINER-SIZE EQUAL ZERO THEN
              MOVE 2 TO SOAPACTION-CONTAINER-SIZE
              MOVE '""' TO SOAPACTION-CONTAINER(1 : 2)
           END-IF.

      * Pass the Basic Authentication header into the pipeline
      * by piggy-backing it on to the end of the SOAPaction header
      * (Note that the SOAPaction header is not written for SOAP 1.2.
      * CICS appends the SOAP action to the Content-Type header
      * "action" parameter instead. This will have the same effect.)

           SET ADDRESS OF SOAPACTION-APPENDAGE TO ADDRESS OF
               SOAPACTION-CHAR(SOAPACTION-CONTAINER-SIZE + 1).

           ADD 17 TO BASIC-AUTH-VALUE-LENGTH GIVING APPEND-LENGTH.
           MOVE BASIC-AUTHENTICATION-HEADER(1 : APPEND-LENGTH)
                TO SOAPACTION-APPENDAGE.

           ADD APPEND-LENGTH TO SOAPACTION-CONTAINER-SIZE.
           EXEC CICS PUT CONTAINER('DFHWS-SOAPACTION') CHAR
                         FROM(SOAPACTION-CONTAINER)
                         FLENGTH(SOAPACTION-CONTAINER-SIZE)
                         END-EXEC.

      * Delete the Response container to allow the pipeline to continue
           EXEC CICS DELETE CONTAINER('DFHRESPONSE')
           END-EXEC.

      * Delete the Request container to allow the pipeline to continue
       DELETE-REQUEST-CONTAINER SECTION.
           EXEC CICS DELETE CONTAINER('DFHREQUEST')
           NOHANDLE END-EXEC.

