CBL CICS('COBOL3') APOST
      *****************************************************************
      *
      *  MODULE NAME = WSSAMPLE
      *
      *  DESCRIPTIVE NAME = Web Service sample to drive basic auth
      *
      *  Licensed Materials - Property of IBM
      *
      *  SupportPac CA8J
      *
      *  (C) Copyright IBM Corporation 2006    All Rights Reserved.
      *
      *  US Government Users Restricted Rights - Use, duplication
      *  or disclosure restricted by GSA ADP Schedule Contract
      *  with IBM Corporation
      *
      *  FUNCTION =
      *     This is a sample program that initiates a Web Service
      *     requester pipeline. It is used to demonstrate the use
      *     of the utilities to insert a Basic Authentication header.
      *
      *  NOTES :
      *   DEPENDENCIES = S/390
      *   RESTRICTIONS =
      *   PATCH LABEL = None
      *   MODULE TYPE = Executable
      *   PROCESSOR = COBOL
      *   ATTRIBUTES = Read only, Serially Reusable
      *
      ******************************************************************
       IDENTIFICATION DIVISION.
       PROGRAM-ID.    WSSAMPLE.
       DATE-WRITTEN.  December 2005.
       DATE-COMPILED. October 2006.
      ******************************************************************
      ****                                                          ****
      ****        B A S I C   S O A P   S A M P L E                 ****
      ****                                                          ****
      ****                                                          ****
      ****  This program is a primitive SOAP requester application  ****
      ****  whose only purpose is to drive the HTTP Basic           ****
      ****  Authentication transport handler.                       ****
      ****                                                          ****
      ******************************************************************

       ENVIRONMENT DIVISION.

       DATA DIVISION.

       WORKING-STORAGE SECTION.
        77 PIPELINE-NAME          PIC X(8)  VALUE 'WSBASIC'.
        77 WEBSERVICE-SOAPACTION  PIC X(4)  VALUE 'None'.
        77 WS-RESPONSE-LENGTH     PIC S9(8) COMP-5 SYNC.
        77 WS-RESPONSE-PTR        USAGE POINTER.
        77 WS-RESPONSE-DATA-PTR   USAGE POINTER.

        01 WEBSERVICE-URL.
           02 FILLER PIC X(7)  VALUE 'http://'.
           02 FILLER PIC X(29) VALUE 'winmvs2c.hursley.ibm.com:5053'.
           02 FILLER PIC X(29) VALUE '/basicwebservice'.

        01 WEBSERVICE-BODY.
           02 FILLER PIC X(15) VALUE '<SOAP-ENV:Body>'.
           02 FILLER PIC X(8)  VALUE '<Symbol>'.
           02 FILLER PIC X(3)  VALUE 'ABC'.
           02 FILLER PIC X(9)  VALUE '</Symbol>'.
           02 FILLER PIC X(16) VALUE '</SOAP-ENV:Body>'.
           02 FILLER PIC X(2)  VALUE X'0D25'.

       LINKAGE SECTION.
        01 WS-RESPONSE-DATA.
           02 FILLER PIC X OCCURS 1 TO 32767
                           DEPENDING ON WS-RESPONSE-LENGTH.

        01 WS-RESPONSE-CONTAINER.
           02 FILLER PIC X OCCURS 1 TO 32767
                           DEPENDING ON WS-RESPONSE-LENGTH.

       PROCEDURE DIVISION.
       MAINLINE SECTION.

      * Populate required containers in channel 'WS'.
           EXEC CICS PUT CHANNEL('WS') CONTAINER('DFHWS-PIPELINE')
                                       CHAR FROM(PIPELINE-NAME)
                                       END-EXEC.
           EXEC CICS PUT CHANNEL('WS') CONTAINER('DFHWS-URI')
                                       CHAR FROM(WEBSERVICE-URL)
                                       END-EXEC.
           EXEC CICS PUT CHANNEL('WS') CONTAINER('DFHWS-BODY')
                                       CHAR FROM(WEBSERVICE-BODY)
                                       END-EXEC.
           EXEC CICS PUT CHANNEL('WS') CONTAINER('DFHWS-SOAPACTION')
                                       CHAR FROM(WEBSERVICE-SOAPACTION)
                                       END-EXEC.

      * Link to pipeline router using channel 'WS'.
           EXEC CICS LINK PROGRAM('DFHPIRT') CHANNEL('WS')
                                             END-EXEC.

      * Get the response
           MOVE ZERO TO WS-RESPONSE-LENGTH.
           EXEC CICS GET CHANNEL('WS') CONTAINER('DFHRESPONSE')
                                       SET(WS-RESPONSE-PTR)
                                       FLENGTH(WS-RESPONSE-LENGTH)
                                       END-EXEC.

           EXEC CICS GETMAIN SET(WS-RESPONSE-DATA-PTR)
                             FLENGTH(WS-RESPONSE-LENGTH)
                             END-EXEC.

           SET ADDRESS OF WS-RESPONSE-CONTAINER TO WS-RESPONSE-PTR.
           SET ADDRESS OF WS-RESPONSE-DATA TO WS-RESPONSE-DATA-PTR.
           MOVE WS-RESPONSE-CONTAINER TO WS-RESPONSE-DATA.

      * Convert line-ends to field-marks, and tabs to spaces:
           INSPECT WS-RESPONSE-DATA
                   CONVERTING X'0D251505' TO X'1E1E1E40'.

      * Display up to 24 lines of the response
           IF WS-RESPONSE-LENGTH > 1920 THEN
              MOVE 1920 TO WS-RESPONSE-LENGTH.

           EXEC CICS SEND ERASE FROM(WS-RESPONSE-DATA)
                                FLENGTH(WS-RESPONSE-LENGTH)
                                END-EXEC.

      * Return
           EXEC CICS RETURN
           END-EXEC.